<?php
if (!defined('ABSPATH')) exit;

require_once DMC_INCLUDES_DIR . '/controls.php';
$controls = new NewsletterControls();
$module = NewsletterEmails::instance();

if ($controls->is_action('convert')) {
    $module->convert_old_emails();
    $controls->messages = 'Converted!';
}

if ($controls->is_action('unconvert')) {
    $wpdb->query("update wp_newsletter_emails set type='email' where type='message'");
    $controls->messages = 'Unconverted!';
}

if ($controls->is_action('send')) {
    $dmc->hook_newsletter();
    $controls->messages .= __('Delivery engine triggered.', 'digital-media-combined');
}

if ($controls->is_action('copy')) {
    $original = DMC::instance()->get_email(sanitize_text_field($_POST['btn']));
    $email = array();
    $email['subject'] = $original->subject;
    $email['message'] = $original->message;
    $email['message_text'] = $original->message_text;
    $email['send_on'] = time();
    $email['type'] = 'message';
    $email['editor'] = $original->editor;
    $email['track'] = $original->track;
    if(isset($original->options)) {
        $original_options = unserialize($original->options);
        $email['options'] = serialize(array('composer' => $original_options['composer']));
    }

    DMC::instance()->save_email($email);
    $controls->messages .= __('Message duplicated.', 'digital-media-combined');
}

if ($controls->is_action('delete')) {
    DMC::instance()->delete_email(sanitize_text_field($_POST['btn']));
    $controls->messages .= __('Message deleted.', 'digital-media-combined');
}

if ($controls->is_action('delete_selected')) {
	$ids = $_POST['ids'];
	array_walk($ids, sanitize_text_field);
    $r = DMC::instance()->delete_email($ids);
    $controls->messages .= $r . ' message(s) deleted';
}

$emails = DMC::instance()->get_emails('message');
?>

<div class="wrap" id="template-wrap">

    <?php include DMC_DIR . '/template-header.php'; ?>

    <div id="template-heading">

        <h2><?php _e('Newsletters', 'digital-media-combined') ?></h2>
	<p>This section enables Newsletters to be created and sent with the option of sending to Social Media profiles at the same time.</p>

    </div>

    <div id="template-body">

        <form method="post" action="">
            <?php $controls->init(); ?>

            <?php if ($module->has_old_emails()) { ?>
                <div class="newsletter-message">
                    <p>
                        Your Newsletter installation has emails still in old format. To get them listed, you should convert them in
                        a new format. Would you to convert them now?
                    </p>
                    <p>
                        <?php $controls->button('convert', 'Convert now'); ?>
                    </p>
                </div>
            <?php } ?>

            <p>
                <a href="<?php echo $module->get_admin_page_url('composer'); ?>" class="button green"><?php _e('Create a New Newsletter', 'digital-media-combined') ?></a>
                <?php $controls->button_confirm('delete_selected', __('Delete selected newsletters', 'digital-media-combined'), __('Proceed?', 'digital-media-combined')); ?>
                <?php $controls->button('send', __('Trigger the delivery engine', 'digital-media-combined')); ?>
            </p>
            <table class="widefat">
                <thead>
                    <tr>
                        <th>&nbsp;</th>
                        <th>Id</th>
                        <th><?php _e('Subject', 'digital-media-combined') ?></th>
                        <th><?php _e('Status', 'digital-media-combined') ?></th>
                        <th><?php _e('Progress', 'digital-media-combined') ?>&nbsp;(*)</th>
                        <th><?php _e('Date', 'digital-media-combined') ?></th>
                        <th><?php _e('Tracking', 'digital-media-combined') ?></th>
                        <th>&nbsp;</th>
                        <th>&nbsp;</th>
                        <th>&nbsp;</th>
                        <th>&nbsp;</th>
                    </tr>
                </thead>

                <tbody>
                    <?php foreach ($emails as $email) {
                    	//print_r($email);
                        $email_options = unserialize($email->options);
                        ?>
                        <tr>
                            <td><input type="checkbox" name="ids[]" value="<?php echo $email->id; ?>"/></td>
                            <td><?php echo $email->id; ?></td>
                            <td><?php
                                if ($email->subject)
                                    echo htmlspecialchars($email->subject);
                                else
                                    echo "Newsletter #" . $email->id;
                                ?>
                            </td>

                            <td>
                                <?php
                                if ($email->status == 'sending') {
                                    if ($email->send_on > time()) {
                                        _e('Scheduled', 'digital-media-combined');
                                    } else {
                                        _e('Sending', 'digital-media-combined');
                                    }
                                } else {
                                    echo $email->status;
                                }
                                ?>
                            </td>
                            <td><?php if ($email->status == 'sent' || $email->status == 'sending') echo $email->sent . ' ' . __('of', 'digital-media-combined') . ' ' . $email->total; ?></td>
                            <td><?php if ($email->status == 'sent' || $email->status == 'sending') echo $module->format_date($email->send_on); ?></td>
                            <td><?php echo $email->track == 1 ? __('Yes', 'digital-media-combined') : __('Yes', 'digital-media-combined'); ?></td>
                            <td><a class="button orange" href="<?php echo $module->get_admin_page_url(is_array($email_options) && array_key_exists('composer', $email_options) && $email_options['composer'] ? 'composer' : 'edit'); ?>&amp;id=<?php echo $email->id; ?>"><i class="fa fa-pencil"></i> <?php _e('Edit', 'digital-media-combined') ?></a></td>
                            <td>
                                <a class="button" href="<?php echo NewsletterStatistics::instance()->get_statistics_url($email->id); ?>"><i class="fa fa-bar-chart"></i> <?php _e('Statistics', 'digital-media-combined') ?></a>
                            </td>
                            <td><?php $controls->button_copy($email->id); ?></td>
                            <td><?php $controls->button_delete($email->id); ?></td>
                        </tr>
<?php } ?>
                </tbody>
                <tfoot>
                    <tr>
                        <td colspan="11">
                            (*) <?php _e('', 'digital-media-combined') ?>
                        </td>
                    </tr>
                </tfoot>
            </table>
        </form>
    </div>

<?php include DMC_DIR . '/template-footer.php'; ?>

</div>
