<?php
if (! defined ( 'ABSPATH' ))
	exit ();
require_once (ABSPATH . WPINC . '/pluggable.php');
class DMC_Api {
	private $_endpoint = 'https://api.digitalmediacombined.com/';
	private $_api_key = '';
	function __construct() {
		$dmc_option = $this->option ();
		
		if (! isset ( $dmc_option ['api_key'] ) or strlen ( $dmc_option ['api_key'] ) < 25 or substr ( $dmc_option ['api_key'], 0, 4 ) != 'API-') {
			if (! isset ( $dmc_option ['key'] ) or strlen ( $dmc_option ['key'] ) < 12) {
				$dmc_option ['key'] = wp_generate_password ( 12, false );
				$this->save_option ( $dmc_option );
			}
			$register = $this->register ( $dmc_option );
			if ($register ['status'] == 'success') {
				$dmc_option ['api_key'] = decrypt ( $register ['api_key'], $dmc_option ['key'] );
				$this->save_option ( $dmc_option );
			}
		}
		
		$this->_api_key = $dmc_option ['api_key'];
	}
	public function getApiKey() {
		return $this->_api_key;
	}
	
	// Register for an API key
	function register($dmc_option) {
		$home_url = home_url ();
		
		$params = array (
				'home_url' => $home_url,
				'key' => $dmc_option ['key'] 
		);
		$result = $this->call ( 'register', $params );
		print_r($result); exit;
		
		return $result;
	}
	function call($task, array $params) {
		$params ['task'] = $task;
		$dmc_option = $this->option ();
		if (strlen ( $dmc_option ['api_key'] ) >= 25) {
			$params ['api_key'] = $dmc_option ['api_key'];
		}
		$content = json_decode ( dmc_curl_post ( $this->getEndpoint (), $params ), 1 );
		if (isset ( $content ['message'] )) {
			$content ['message'] = str_ireplace ( array (
					'{admin_url}',
					'{plugin_url}' 
			), array (
					DMC_ADMIN_URL,
					DMC_URL 
			), $content ['message'] );
		}
		return $content;
	}
	function getEndpoint() {
		return $this->_endpoint;
	}
	function option() {
		return DMC_option::getInstance ();
	}
	function save_option($dmc_option) {
		return DMC_option::save ( $dmc_option );
	}
}
class DMC_option {
	private static $dmc_option = null;
	private static $option_name = 'dmc_option';
	private function __construct() {
	}
	public static function getInstance() {
		if (self::$dmc_option == null) {
			self::$dmc_option = json_decode ( get_option ( self::$option_name ), 1 );
		}
		
		return self::$dmc_option;
	}
	public static function save($dmc_option) {
		foreach ( $dmc_option as $k => $v ) {
			self::$dmc_option [$k] = $v;
		}
		update_option ( self::$option_name, json_encode ( self::$dmc_option ), 'no' );
	}
}

// End Point
add_action ( 'rest_api_init', function () {
	register_rest_route ( DMC_SLUG . '/v1', '/endpoint/', array (
			'methods' => WP_REST_Server::CREATABLE,
			'callback' => 'dmc_api_func',
			'permission_callback' => 'dmc_permission_check' 
	) );
	
	register_rest_route ( DMC_SLUG . '/v1', '/news/(?P<id>\d+)', array (
			'methods' => WP_REST_Server::READABLE,
			'callback' => 'dmc_news_redirect',
			'permission_callback' => 'dmc_permission_check' 
	) );
} );
function dmc_permission_check($access) {
	if (is_user_logged_in ()) {
		return $access;
	}
	
	if (strpos ( $_SERVER ['REMOTE_ADDR'], '203.132.29.' )) {
		return false;
	}
	return true;
}
add_filter ( 'rest_authentication_errors', 'dmc_permission_check', 20 );
function dmc_news_redirect(WP_REST_Request $request) {
	$parameters = $request->get_params ();
	$url = get_site_url () . '/?na=v&id=' . $parameters ['id'];
	wp_redirect ( $url );
	exit ();
}

// End Point Callback
function dmc_api_func(WP_REST_Request $request) {
	$parameters = $request->get_params ();
	$parameters ['status'] = 'error';
	
	if (isset ( $parameters ['api_key'] )) {
		$api_key = sanitize_text_field ( $parameters ['api_key'] );
		$dmc_option = DMC_option::getInstance ();
		$api_key = dmc_decrypt ( $api_key, $dmc_option ['key'] );
		if (substr ( $api_key, 0, 4 ) == 'API-') {
			$parameters ['status'] = 'success';
			$dmc_option ['api_key'] = $api_key;
			DMC_option::save ( $dmc_option );
			$parameters ['api_key'] = $api_key;
			$parameters ['message'] = 'Successfully obtain API Key';
		} else {
			$parameters ['status'] = 'error';
			$parameters ['api_key'] = $api_key;
			$parameters ['message'] = 'Unable to obtain API key';
		}
	}
	
	return wp_send_json ( $parameters );
}

// Encrypt Function

function dmc_encrypt($plaintext, $key) {
	// $plaintext = "message to be encrypted";
	$ivlen = openssl_cipher_iv_length ( $cipher = "AES-128-CBC" );
	$iv = openssl_random_pseudo_bytes ( $ivlen );
	$ciphertext_raw = openssl_encrypt ( $plaintext, $cipher, $key, $options = OPENSSL_RAW_DATA, $iv );
	$hmac = hash_hmac ( 'sha256', $ciphertext_raw, $key, $as_binary = true );
	$ciphertext = base64_encode ( $iv . $hmac . $ciphertext_raw );
	return $ciphertext;
}

// Decrypt Function

function dmc_decrypt($encrypted, $key) {
	$key = trim ( $key );
	$c = base64_decode ( $encrypted );
	$ivlen = openssl_cipher_iv_length ( $cipher = "AES-128-CBC" );
	$iv = substr ( $c, 0, $ivlen );
	$hmac = substr ( $c, $ivlen, $sha2len = 32 );
	$ciphertext_raw = substr ( $c, $ivlen + $sha2len );
	$original_plaintext = openssl_decrypt ( $ciphertext_raw, $cipher, $key, $options = OPENSSL_RAW_DATA, $iv );
	$calcmac = hash_hmac ( 'sha256', $ciphertext_raw, $key, $as_binary = true );
	if (hash_equals ( $hmac, $calcmac )) 
	{
		return trim($original_plaintext);
	}
}

// Post using curl instead of wp_remote_post so we can set the IP to the domain's instead of the server's
function dmc_curl_post($url, $params) {
	if (isset ( $_SERVER ['SERVER_ADDR'] )) {
		$server_ip = $_SERVER ['SERVER_ADDR'];
	} elseif ($_SERVER ['LOCAL_ADDR']) {
		$server_ip = $_SERVER ['LOCAL_ADDR'];
	} else {
		return false;
	}
	
	$ch = curl_init ();
	curl_setopt ( $ch, CURLOPT_URL, $url );
	curl_setopt ( $ch, CURLOPT_POST, 1 );
	curl_setopt ( $ch, CURLOPT_RETURNTRANSFER, true );
	curl_setopt ( $ch, CURLOPT_POSTFIELDS, http_build_query ( $params ) );
	curl_setopt ( $ch, CURLOPT_INTERFACE, $server_ip );
	$output = curl_exec ( $ch );
	
	curl_close ( $ch );
	return $output;
}