<?php
if (!defined('ABSPATH')) exit;

@include_once DMC_INCLUDES_DIR . '/controls.php';
$module = DMC::instance();
$controls = new NewsletterControls();

if (!$controls->is_action()) {
    $controls->data = $module->get_options('smtp');
} else {


    if ($controls->is_action('save')) {

        if ($controls->data['enabled'] && empty($controls->data['host'])) {
            $controls->errors = 'The host must be set to enable the SMTP';
        }

        if (empty($controls->errors)) {
            $module->save_options($controls->data, 'smtp');
            $controls->messages .= __('Saved. Remember to test your changes right now!', 'digital-media-combined');
        }
    }

    if ($controls->is_action('test')) {

        require_once ABSPATH . WPINC . '/class-phpmailer.php';
        require_once ABSPATH . WPINC . '/class-smtp.php';
        $mail = new PHPMailer();
        ob_start();
        $mail->IsSMTP();
        $mail->SMTPDebug = true;
        $mail->CharSet = 'UTF-8';
        $message = 'This Email is sent by PHPMailer of WordPress';
        $mail->IsHTML(false);
        $mail->Body = $message;
        $mail->From = $module->options['sender_email'];
        $mail->FromName = $module->options['sender_name'];
        if (!empty($module->options['return_path'])) {
            $mail->Sender = $module->options['return_path'];
        }
        if (!empty($module->options['reply_to'])) {
            $mail->AddReplyTo($module->options['reply_to']);
        }

        $mail->Subject = '[' . get_option('blogname') . '] SMTP test';

        $mail->Host = $controls->data['host'];
        if (!empty($controls->data['port'])) {
            $mail->Port = (int) $controls->data['port'];
        }

        $mail->SMTPSecure = $controls->data['secure'];
        $mail->SMTPAutoTLS = false;
        
        if ($controls->data['ssl_insecure'] == 1) {
                $mail->SMTPOptions = array(
                    'ssl' => array(
                        'verify_peer' => false,
                        'verify_peer_name' => false,
                        'allow_self_signed' => true
                    )
                );
        }

        if (!empty($controls->data['user'])) {
            $mail->SMTPAuth = true;
            $mail->Username = $controls->data['user'];
            $mail->Password = $controls->data['pass'];
        }

        $mail->SMTPKeepAlive = true;
        $mail->ClearAddresses();
        $mail->AddAddress($controls->data['test_email']);

        $mail->Send();
        $mail->SmtpClose();
        $debug = htmlspecialchars(ob_get_clean());

        if ($mail->IsError()) {
            $controls->errors = '<strong>Connection/email delivery failed.</strong><br>You should contact your provider reporting the SMTP parameter and asking about connection to that SMTP.<br><br>';
            $controls->errors = $mail->ErrorInfo;
        } else
            $controls->messages = 'Success.';

        $controls->messages .= '<textarea style="width:100%; height:200px; font-size:12px; font-family: monospace">';
        $controls->messages .= $debug;
        $controls->messages .= '</textarea>';
    }
}
?>

<div class="wrap" id="template-wrap">

    <?php include DMC_DIR . '/template-header.php'; ?>

	<div id="template-heading">

        <h2><?php _e('SMTP Settings', 'digital-media-combined') ?></h2>
    
    <p>
        <a href="https://www.digitalmediacombined.com/packages" target="_blank">Upgrade your SMTP services and use Digital Media Combined</a>
    </p>
    
    <p>
            <strong>Let Digital Media Combined manage the delivery of your emails through our industrial server network. Eliminate the chance of your domain being blacklisted!</strong><br />
		DMC allows up to 1000 emails to be sent to under 1000 subscribers for free each month. Delivery queue limits apply.<br />
		<a href="https://www.digitalmediacombined.com/packages" target="_blank">Over 1000 Subscribers and need to send more emails? Remove restrictions! Click here to view our packages</a>
        </p>
    
    </div>

	<div id="template-body">

    <form method="post" action="">
        <?php $controls->init(); ?>

        <table class="form-table">
            <tr>
                <th>Enable the SMTP?</th>
                <td><?php $controls->yesno('enabled'); ?></td>
            </tr>
            <tr>
                <th>SMTP host/port</th>
                <td>
                    Host: <?php $controls->text('host', 30); ?>
                    Port: <?php $controls->text('port', 6); ?>
                    <?php $controls->select('secure', array('' => 'No secure protocol', 'tls' => 'TLS protocol', 'ssl' => 'SSL protocol')); ?>
                    <p class="description">
                        To use Gmail try host "smtp.gmail.com" and port 465 and SSL protocol
                    </p>
                </td>
            </tr>
            <tr>
                <th>Authentication</th>
                <td>
                    User: <?php $controls->text('user', 30); ?>
                    Password: <?php $controls->text('pass', 30); ?>
                    <p class="description">
                        If user authentication is not required, leave blank.
                    </p>
                </td>
            </tr>
            <tr>
                <th>Insecure SSL Connections</th>
                <td>
                    <?php $controls->yesno('ssl_insecure'); ?> 
                </td>
            </tr>
            <tr>
                <th>Test email address</th>
                <td>
                    <?php $controls->text_email('test_email', 30); ?>
                    <?php $controls->button('test', 'Send a test email to this address'); ?>
                    <p class="description">
                        If the test reports a "connection failed", review your settings.  If correct your current provider may not allow for mail to be sent this way.  <a href="https://www.digitalmediacombined.com/packages" target="_blank">Click here to view Digital Media Combined packages</a>
                    </p>
                </td>
            </tr>
        </table>

        <p>
            <?php $controls->button_save(); ?>
        </p>

    </form>
</div>

    <?php include DMC_DIR . '/template-footer.php'; ?>
    
</div>
