<?php
/* @var $wpdb wpdb */
if (!defined('ABSPATH')) exit;

require_once DMC_INCLUDES_DIR . '/controls.php';
$module = NewsletterStatistics::instance();
$controls = new NewsletterControls();

if ($controls->is_action('country')) {
    $module->country();
    $controls->messages = $module->country_result;
}

if ($controls->is_action('import')) {
    $wpdb->query("insert ignore into " . $wpdb->prefix . "newsletter_sent (user_id, email_id, time) select user_id, email_id, UNIX_TIMESTAMP(created) from " . DMC_STATS_TABLE);
    $controls->messages = 'Done!';
}

$types = $wpdb->get_results("select distinct type from " . DMC_EMAILS_TABLE);
$type_options = array();

foreach ($types as $type) {
    if ($type->type == 'followup')
        continue;
    if ($type->type == 'message') {
        $type_options[$type->type] = 'Standard Newsletter';
    } else if ($type->type == 'feed') {
        $type_options[$type->type] = 'Feed by Mail';
    } else if (strpos($type->type, 'automated') === 0) {
        list($a, $id) = explode('_', $type->type);
        $type_options[$type->type] = 'Automated Channel ' . $id;
    } else {
        $type_options[$type->type] = $type->type;
    }
}


if (empty($controls->data['type'])) {
    $emails = $wpdb->get_results("select send_on, id, subject, total, status, type, track, sent, subject from " . DMC_EMAILS_TABLE . " where status='sent' order by send_on desc limit 20");
} else {
    $emails = $wpdb->get_results($wpdb->prepare("select send_on, id, subject, total, type from " . DMC_EMAILS_TABLE . " where status='sent' and type=%s order by send_on desc limit 20", $controls->data['type']));
}
$overview_labels = array();
$overview_titles = array();
$overview_open_rate = array();
$overview_click_rate = array();

$total_sent = 0;
$open_count_total = 0;
$click_count_total = 0;
foreach ($emails as $email) {
    $entry = array();
    $total_sent += $email->sent;
    if (empty($email->sent)) {
//        $entry[0] = date('Y-m-d', $email->send_on);
//        $entry[1] = 0;
//	$entry[2] = $email->subject;// . ' (' . percent($open_count, $email->sent) . ')';
//        $entry[3] = 0;
        continue;
    }
    //$entry[0] = $email->subject . ' [' . date('Y-m-d', $email->send_on) . ', ' . $email->type . ']';
    $entry[0] = date('Y-m-d', $email->send_on);
    $open_count = $wpdb->get_var("select count(distinct user_id) from " . DMC_STATS_TABLE . " where email_id=" . $email->id);
    $open_count_total += $open_count;
    $entry[1] = $open_count / $email->sent * 100;
    $entry[1] = round($entry[1], 2);
    $entry[2] = $email->subject; // . ' (' . percent($open_count, $email->sent) . ')';
    $click_count = $wpdb->get_var("select count(distinct user_id) from " . DMC_STATS_TABLE . " where url<>'' and email_id=" . $email->id);
    $click_count_total += $click_count;
    $entry[3] = $click_count / $email->sent * 100;
    $entry[3] = round($entry[3], 2);

    $overview_labels[] = strftime('%a, %e %b', $email->send_on);
    $overview_open_rate[] = $entry[1];
    $overview_click_rate[] = $entry[3];
    $overview_titles[] = $entry[2];
}

$overview_labels = array_reverse($overview_labels);
$overview_open_rate = array_reverse($overview_open_rate);
$overview_click_rate = array_reverse($overview_click_rate);

function percent($value, $total) {
    if ($total == 0)
        return '-';
    return sprintf("%.2f", $value / $total * 100) . '%';
}

function percentValue($value, $total) {
    if ($total == 0)
        return 0;
    return round($value / $total * 100);
}
?>

<script type="text/javascript" src="<?php echo plugins_url('../js/Chart2.min.js', __FILE__) ?>"></script>
<script type="text/javascript" src="<?php echo plugins_url('../js/jquery.vmap.min.js', __FILE__) ?>"></script>
<script type="text/javascript" src="<?php echo plugins_url('../js/jquery.vmap.world.js', __FILE__) ?>"></script>
<link href="<?php echo plugins_url('../css/jqvmap.cs', __FILE__) ?>" media="screen" rel="stylesheet" type="text/css"/>

<div class="wrap" id="template-wrap">
    <?php include DMC_DIR . '/template-header.php' ?>
    <div id="template-heading">
        
        
        <h2><?php _e('Global Newsletter Statistics', 'digital-media-combined') ?></h2>
            
    </div>

    <div id="template-body">
        <form method="post" action="">

            <?php $controls->init(); ?>

            <?php if (empty($emails)) { ?>
               
                
            <?php } else { ?>
                
                <div class="row">
                    
                        <div class="template-statistics-info-box">
                            <p class="template-legend">Select Newsletter category:</p>
                            <?php $controls->select('type', $type_options, 'All') ?>
                            <?php $controls->button('update', __('Update Charts', 'digital-media-combined')) ?>
                        </div>
                    
                </div>

                <br>

                <div class="row">
                    <div class="col-md-6">
                        <div class="template-widget">
                            <h3>Overview (Last 20 Newsletters)</h3>
                            <div class="inside">
                                
                                <p class="template-events-legend">Subscribers interactions distribution over time,<br>starting from the sending day.</p>
                                
                                <div id="template-events-chart">
                                    <canvas id="template-events-chart-canvas"></canvas>
                                </div>

                                <script type="text/javascript">
                                    var events_data = {
                                        labels: <?php echo json_encode($overview_labels) ?>,
                                        datasets: [
                                            {
                                                label: "Open",
                                                fill: false,
                                                strokeColor: "#27AE60",
                                                backgroundColor: "#27AE60",
                                                borderColor: "#27AE60",
                                                pointBorderColor: "#27AE60",
                                                pointBackgroundColor: "#27AE60",
                                                data: <?php echo json_encode($overview_open_rate) ?>
                                            },
                                            {
                                                label: "Click",
                                                fill: false,
                                                strokeColor: "#C0392B",
                                                backgroundColor: "#C0392B",
                                                borderColor: "#C0392B",
                                                pointBorderColor: "#C0392B",
                                                pointBackgroundColor: "#C0392B",
                                                data: <?php echo json_encode($overview_click_rate) ?>,
                                                yAxisID: "y-axis-2"
                                            }
                                        ]
                                    };
                                    
                                    var titles = <?php echo json_encode(array_reverse($overview_titles)) ?>;

                                    jQuery(document).ready(function ($) {
                                        ctxe = $('#template-events-chart-canvas').get(0).getContext("2d");
                                        eventsLineChart = new Chart(ctxe, {type: 'line', data: events_data,
                                            options: {
                                                scales: {
                                                    xAxes: [{type:"category","id":"x-axis-1", gridLines: {display: false}, ticks: {fontFamily: "Source Sans Pro"}}],
                                                    yAxes: [
                                                        {type: "linear", "id": "y-axis-1", gridLines: {display: false}, ticks: {fontColor: "#27AE60", fontFamily: "Source Sans Pro"}},
                                                        {type: "linear", "id": "y-axis-2", position: "right", gridLines: {display: false}, ticks: {fontColor: "#C0392B", fontFamily: "Source Sans Pro"}}
                                                    ]
                                                },
                                                tooltips: {
                                                    callbacks: {
                                                        afterTitle: function(data) {
                                                            return titles[data[0].index];
                                                        },
                                                        label: function(tooltipItem, data) {
                                                            return data.datasets[0].label + ": " + data.datasets[0].data[tooltipItem.index] + "% " + 
                                                                    data.datasets[1].label + ": " + data.datasets[1].data[tooltipItem.index] + "%";
                                                        }
                                                    }
                                                }
                                            }
                                        });
                                    });
                                </script>
                                
                                <div class="row">
                                    <div class="col-md-3">
                                        <div class="template-data">
                                            <div class="template-data-title"><?php _e('Total Sent Messages', 'digital-media-combined') ?></div>
                                            <div class="template-data-value"><?php echo $total_sent; ?></div>
                                        </div>
                                    </div>

                                    <div class="col-md-3">
                                        <div class="template-data">
                                            <div class="template-data-title"><?php _e('Total Interactions', 'digital-media-combined') ?></div>
                                            <div class="template-data-value"><?php echo $open_count_total; ?> (<?php echo percent($open_count_total, $total_sent); ?>)</div>
                                        </div>
                                    </div>

                                    <div class="col-md-3">
                                        <div class="template-data">
                                            <div class="template-data-title"><?php _e('Opened Newsletters', 'digital-media-combined') ?></div>
                                            <div class="template-data-value"><?php echo $open_count_total - $click_count_total; ?> (<?php echo percent($open_count_total - $click_count_total, $total_sent); ?>)</div>
                                        </div>
                                    </div>

                                    <div class="col-md-3">
                                        <div class="template-data">
                                            <div class="template-data-title"><?php _e('Clicked Newsletters', 'digital-media-combined') ?></div>
                                            <div class="template-data-value"><?php echo $click_count_total; ?> (<?php echo percent($click_count_total, $total_sent); ?>)</div>
                                        </div>
                                    </div>
                                </div>  
                            </div>
                        </div>
                    </div>

                    
                </div>
                
                <div class="row">


                    <!-- LAST NEWSLETTERS -->
                    <div class="col-md-12">
                        <div class="template-widget">
                            <h3><?php _e('Last newsletters', 'digital-media-combined') ?> <a href="admin.php?page=newsletter_statistics_newsletters"><?php _e('Details', 'digital-media-combined') ?></a></h3>
                            <div class="inside">
                                <?php
                                $emails = $wpdb->get_results($wpdb->prepare("select send_on, id, subject, total, status, type, track, sent, subject from " . DMC_EMAILS_TABLE . " where status in ('sent', 'sending') and send_on<%d order by send_on desc limit 5", time()));
                                ?>

                                <table class="widefat">
                                       <thead>
                                        <tr>
                                            <th>Id</th>
                                            <th><?php _e('Subject', 'digital-media-combined') ?></th>
                                            <th>Type</th>
                                            <th><?php _e('Status', 'digital-media-combined') ?></th>
                                            <th>&nbsp;</th>
                                        </tr>
                                    </thead>

                                    <tbody>
                                        <?php foreach ($emails as &$email) { ?>
                                            <tr>
                                                <td><?php echo $email->id; ?></td>
                                                <td><?php echo htmlspecialchars($email->subject); ?></td>
                                                <td><?php echo $module->get_email_type_label($email); ?></td>
                                                <td><?php echo $module->get_email_status_label($email); ?></td>
                                                <td>
                                                    <a href="<?php echo NewsletterStatistics::instance()->get_statistics_url($email->id); ?>" class="button">Statistics</a>
                                                </td>
                                            </tr>
                                        <?php } ?>
                                    </tbody>
                                </table>

                            </div>
                        </div>
                    </div>
                </div>
                
<?/**
                <div class="row">
                    <div class="col-md-6">
                        <div class="template-statistics-info-box">
                            <p class="template-legend">Check Statistics global<br>configurations.</p>
                            <a class="button" href="admin.php?page=newsletter_statistics_settings"><?php _e('Settings') ?></a>
                        </div>
                    </div>
                    <div class="col-md-6">
                        
                    </div>  
                    
                </div>
**/?>
                


            <?php } ?>


        </form>

        
    </div>
    <?php include DMC_DIR . '/template-footer.php' ?>
</div>
