<?php
if (!defined('ABSPATH'))
    exit;

@include_once DMC_INCLUDES_DIR . '/controls.php';
$controls = new NewsletterControls();
$module = NewsletterSubscription::instance();

if (!$controls->is_action()) {
    $controls->data = $module->get_options('template');
} else {
    if ($controls->is_action('save')) {
        $module->save_options($controls->data, 'template');

        if (strpos($controls->data['template'], '{message}') === false) {
            $controls->errors = __('The tag {message} is missing in your template', 'digital-media-combined');
        }

        $controls->add_message_saved();
    }
    if ($controls->is_action('reset')) {
        $controls->data['template'] = file_get_contents(dirname(__FILE__) . '/email.html');
        $controls->add_message_done();
    }

    if ($controls->is_action('test')) {

        $users = NewsletterUsers::instance()->get_test_users();
        if (count($users) == 0) {
            $controls->errors = __('No test subscribers found.', 'digital-media-combined') . ' ';
        } else {
            $template = $controls->data['template'];
            if (strpos($template, '{message}') === false) {
                $template .= '{message}';
            }
            $message = '<p>This is a generic example of message embedded inside the template.</p>';
            $message .= '<p>Subscriber data can be referenced by messages with tags. See the <a href="https://www.digitalmediacombined.com">plugin documentation</a>.</p>';
            $message .= '<p>First name: {name}</p>';
            $message .= '<p>Last name: {surname}</p>';
            $message .= '<p>Email: {email}</p>';

            $message = str_replace('{message}', $message, $template);
            $addresses = array();
            foreach ($users as &$user) {
                $addresses[] = $user->email;
                DMC::instance()->mail($user->email, 'Newsletter Messages Template Test', $dmc->replace($message, $user));
            }
            $controls->messages .= 'Test emails sent to ' . count($users) . ' test subscribers: ' .
                    implode(', ', $addresses) . '.' . ' ';
        }
    }
}
?>
<!--
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/codemirror.css" type="text/css">
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/addon/hint/show-hint.css">
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/codemirror.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/mode/xml/xml.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/mode/css/css.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/mode/javascript/javascript.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/mode/htmlmixed/htmlmixed.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/addon/hint/show-hint.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/addon/hint/xml-hint.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.20.2/addon/hint/html-hint.js"></script>
<script>
    jQuery(function () {
        var editor = CodeMirror.fromTextArea(document.getElementById("options-template"), {
            lineNumbers: true,
            mode: 'htmlmixed',
            extraKeys: {"Ctrl-Space": "autocomplete"}
        });
    });
</script>
-->
<div class="wrap" id="template-wrap">

    <?php include DMC_DIR . '/template-header.php'; ?>

    <div id="template-heading">

        <h2><?php _e('Messages template', 'digital-media-combined') ?></h2>
        <p>
            Edit the default template of confirmation, welcome and cancellation emails. Add the {message} tag where you
            want the specific message text to be included.
        </p>

    </div>

    <div id="template-body">

        <form method="post" action="">
            <?php $controls->init(); ?>
            <p>
                <?php $controls->button_save(); ?>
            </p>
            <table class="form-table">
                <tr valign="top">
                    <th>Enabled?</th>
                    <td>
                        <?php $controls->yesno('enabled'); ?>
                        <p class="description">
                            When not enabled, the old templating system is used (see the file
                            WP_CONTENT_DIR/plugins/<?php echo DMC_SLUG;?>/subscription/email.php).
                        </p>
                    </td>
                </tr>
                <tr valign="top">
                    <th>Email template</th>
                    <td>
                        <?php $controls->textarea_preview('template', '100%', '700'); ?>
                        <?php $controls->button_reset(); ?>
                        <?php $controls->button('test', 'Send a test'); ?>
                    </td>
                </tr>
            </table>
            <p>
                <?php $controls->button_save(); ?>
            </p>
        </form>
    </div>

    <?php include DMC_DIR . '/template-footer.php'; ?>

</div>